import time
import logging
import sys
import tomli

import cli.args
import cli.logger
import workflow.constants
import workflow.manager
import workflow.common
import workflow.utils
import workflow.executor


def start():
    # parse arguments
    args = cli.args.parse()
    # configure logger
    log_config = cli.logger.LogConfiguration(logging.getLevelName(args.log_level), args.log_rotation,
                                             args.log_file_size, args.log_file_name)
    logger = cli.logger.configure(log_config)
    try:
        with open(args.env_config, "rb") as fp:
            loader_config = tomli.load(fp)
    except OSError as e:
        logger.exception("Failed to load 'loader.toml' config.", e)
        sys.exit(1)
    loader_config[workflow.constants.RAI_SDK_HTTP_RETRIES] = args.rai_sdk_http_retries
    # init env config
    env_config = workflow.common.EnvConfig.from_env_vars(loader_config)
    # init Workflow resource manager
    resource_manager = workflow.manager.ResourceManager.init(logger, args.engine, args.database, env_config)
    logger.info("Using: " + ",".join(f"{k}={v}" for k, v in vars(args).items()))
    try:
        logger.info(f"Activating batch with config from '{args.batch_config}'")
        start_time = time.time()
        # load batch config as json string
        batch_config_json = workflow.utils.read(args.batch_config)
        # create engine if it doesn't exist
        resource_manager.add_engine(args.engine_size)
        # Skip infrastructure setup during recovery
        if not args.recover and not args.recover_step:
            # Create db and disable IVM in case of enabled flag
            resource_manager.create_database(args.drop_db, args.disable_ivm, args.source_database)
        # Init workflow executor
        parameters = {
            workflow.constants.REL_CONFIG_DIR: args.rel_config_dir,
            workflow.constants.START_DATE: args.start_date,
            workflow.constants.END_DATE: args.end_date,
            workflow.constants.FORCE_REIMPORT: args.force_reimport,
            workflow.constants.FORCE_REIMPORT_NOT_CHUNK_PARTITIONED: args.force_reimport_not_chunk_partitioned,
            workflow.constants.COLLAPSE_PARTITIONS_ON_LOAD: args.collapse_partitions_on_load
        }
        config = workflow.executor.WorkflowConfig(env_config, workflow.common.BatchConfig(args.batch_config_name,
                                                                                          batch_config_json),
                                                  args.recover, args.recover_step, args.selected_steps, parameters,
                                                  args.step_timeout_dict)
        executor = workflow.executor.WorkflowExecutor.init(logger, config, resource_manager)
        end_time = time.time()
        executor.run()
        # Print execution time information
        executor.print_timings()
        logger.info(f"Infrastructure setup time is {workflow.utils.format_duration(end_time - start_time)}")
    except Exception as e:
        # Cleanup resources in case of any failure.
        logger.exception(e)
        sys.exit(1)
    finally:
        if args.cleanup_resources:
            resource_manager.cleanup_resources()
        else:
            if args.cleanup_db:
                resource_manager.delete_database()
            if args.cleanup_engine:
                resource_manager.cleanup_engines()




import glob
import logging
import os.path
import pathlib
from typing import List

from workflow import blob, constants
from workflow.common import EnvConfig, AzureConfig, LocalConfig, SnowflakeConfig, Container, ContainerType, FileMetadata


class PathsBuilder:

    def build(self, logger: logging.Logger, days: List[str], relative_path: str, input_format,
              is_date_partitioned: bool) -> List[FileMetadata]:
        paths = self._build(logger, days, relative_path, input_format, is_date_partitioned)
        if not paths:
            logger.warning(f"""PathsBuilder didn't find any file for specified parameters:
            days=`{days}`, relative_path=`{relative_path}`, input_format=`{input_format}`, 
            is_date_partitioned=`{is_date_partitioned}`""")
        return paths

    def _build(self, logger: logging.Logger, days: List[str], relative_path: str, input_format,
               is_date_partitioned: bool) -> List[FileMetadata]:
        raise NotImplementedError("This class is abstract")


class LocalPathsBuilder(PathsBuilder):
    config: LocalConfig

    def __init__(self, config):
        self.config = config

    def _build(self, logger: logging.Logger, days: List[str], relative_path, extensions: List[str],
               is_date_partitioned: bool) -> List[FileMetadata]:
        paths = []
        files_path = f"{self.config.data_path}/{relative_path}"
        if is_date_partitioned:
            for day in days:
                folder_path = f"{files_path}/{constants.DATE_PREFIX}{day}"
                day_paths = [FileMetadata(os.path.abspath(path), os.path.getsize(path), day) for path in
                             self._get_folder_paths(folder_path, extensions)]
                paths.extend(day_paths)
        else:
            paths = [FileMetadata(os.path.abspath(path), os.path.getsize(path)) for path in
                     self._get_folder_paths(files_path, extensions)]
        return paths

    @staticmethod
    def _get_folder_paths(folder_path: str, extensions: List[str]):
        paths = []
        if pathlib.Path(folder_path).is_dir():
            for ext in extensions:
                paths.extend(glob.glob(f"{folder_path}/*.{ext}"))
        return paths


class AzurePathsBuilder(PathsBuilder):
    config: AzureConfig

    def __init__(self, config: AzureConfig):
        self.config = config

    def _build(self, logger: logging.Logger, days: List[str], relative_path, extensions: List[str],
               is_date_partitioned: bool) -> List[FileMetadata]:
        import_data_path = self.config.data_path
        files_path = f"{import_data_path}/{relative_path}"

        logger.info(f"Loading data from blob import path: '{import_data_path}'")

        paths = []
        if is_date_partitioned:
            for day in days:
                logger.debug(f"Day from range: {day}")
                day_paths = blob.list_files_in_containers(logger, self.config,
                                                          f"{files_path}/{constants.DATE_PREFIX}{day}")
                for path in day_paths:
                    path.as_of_date = day
                paths += day_paths
        else:
            paths = blob.list_files_in_containers(logger, self.config, files_path)
        return paths


class SnowflakePathsBuilder(PathsBuilder):
    config: SnowflakeConfig

    def __init__(self, config: SnowflakeConfig):
        self.config = config

    def _build(self, logger: logging.Logger, days: List[str], relative_path, extensions: List[str],
               is_date_partitioned: bool) -> List[FileMetadata]:
        return [FileMetadata(path=f"{self.config.database}.{self.config.schema}.{relative_path}")]


class PathsBuilderFactory:
    __CONTAINER_TYPE_TO_BUILDER = {
        ContainerType.LOCAL: lambda container: LocalPathsBuilder(EnvConfig.get_config(container)),
        ContainerType.AZURE: lambda container: AzurePathsBuilder(EnvConfig.get_config(container)),
        ContainerType.SNOWFLAKE: lambda container: SnowflakePathsBuilder(EnvConfig.get_config(container)),
    }

    @staticmethod
    def get_path_builder(container: Container) -> PathsBuilder:
        return PathsBuilderFactory.__CONTAINER_TYPE_TO_BUILDER[container.type](container)
